@@ -1,6 +1,6 @@
 package Text::CSV_XS;
 
-# Copyright (c) 2007-2013 H.Merijn Brand.  All rights reserved.
+# Copyright (c) 2007-2014 H.Merijn Brand.  All rights reserved.
 # Copyright (c) 1998-2001 Jochen Wiedmann. All rights reserved.
 # Copyright (c) 1997 Alan Citterman.       All rights reserved.
 #
@@ -27,7 +27,7 @@ use DynaLoader ();
 use Carp;
 
 use vars   qw( $VERSION @ISA );
-$VERSION = "1.01";
+$VERSION = "1.03";
 @ISA     = qw( DynaLoader );
 bootstrap Text::CSV_XS $VERSION;
 
@@ -60,6 +60,7 @@ my %def_attr = (
     quote_null			=> 1,
     quote_binary		=> 1,
     binary			=> 0,
+    decode_utf8			=> 1,
     keep_meta_info		=> 0,
     allow_loose_quotes		=> 0,
     allow_loose_escapes		=> 0,
@@ -156,6 +157,7 @@ my %_cache_id = ( # Only expose what is accessed from within PM
     quote_space			=> 25,
     quote_null			=> 31,
     quote_binary		=> 32,
+    decode_utf8			=> 35,
     _is_bound			=> 26,	# 26 .. 29
     );
 
@@ -264,6 +266,13 @@ sub binary
     $self->{binary};
     } # binary
 
+sub decode_utf8
+{
+    my $self = shift;
+    @_ and $self->_set_attr_X ("decode_utf8", shift);
+    $self->{decode_utf8};
+    } # decode_utf8
+
 sub keep_meta_info
 {
     my $self = shift;
@@ -616,6 +625,70 @@ sub print_hr
     $self->print ($io, [ map { $hr->{$_} } $self->column_names ]);
     } # print_hr
 
+sub fragment
+{
+    my ($self, $io, $spec) = @_;
+
+    my $qd = qr{\s* [0-9]+ \s* }x;
+    my $qr = qr{$qd (?: - (?: $qd | \s* \* \s* ))?}x;
+    my $qc = qr{$qr (?: ; $qr)*}x;
+    defined $spec && $spec =~ m{^ \s*
+	\x23 ? \s*			# optional leading #
+	( row | col | cell ) \s* =
+	( $qc				# for row and col
+	| $qd , $qd (?: - $qd , $qd)?	# for cell
+	) \s* $}xi or croak ($self->SetDiag (2014));
+    my ($type, $range) = (lc $1, $2);
+
+    my @c;
+    if ($type eq "cell") {
+	my ($tlr, $tlc, $brr, $brc) = ($range =~ m{
+	    ^ \s*
+		([0-9]+) \s* , \s* ([0-9]+)
+	    \s* (?: - \s*
+		([0-9]+) \s* , \s* ([0-9]+)
+		)?
+	    \s* $}x) or croak ($self->SetDiag (2014));
+	defined $brr or ($brr, $brc) = ($tlr, $tlc);
+	$tlr <= 0 || $tlc <= 0 || $brr <= 0 || $brc <= 0 ||
+	    $brr < $tlr || $brc < $tlc and croak ($self->SetDiag (2014));
+	$_-- for $tlc, $brc;
+	my $r = 0;
+	while (my $row = $self->getline ($io)) {
+	    ++$r <  $tlr and next;
+	    push @c, [ @{$row}[$tlc..$brc] ];
+	      $r >= $brr and last;
+	    }
+	return \@c;
+	}
+
+    # row or col
+    my @r;
+    my $eod = 0;
+    for (split m/\s*;\s*/ => $range) {
+	my ($from, $to) = m/^\s* ([0-9]+) (?: \s* - \s* ([0-9]+ | \* ))? \s* $/x
+	    or croak ($self->SetDiag (2014));
+	$to ||= $from;
+	$to eq "*" and ($to, $eod) = ($from, 1);
+	$from <= 0 || $to <= 0 || $to < $from and croak ($self->SetDiag (2014));
+	$r[$_] = 1 for $from .. $to;
+	}
+
+    my $r = 0;
+    $type eq "col" and shift @r;
+    $_ ||= 0 for @r;
+    while (my $row = $self->getline ($io)) {
+	$r++;
+	if ($type eq "row") {
+	    ($r > $#r && $eod) || $r[$r] and push @c, $row;
+	    next;
+	    }
+	push @c, [ map { ($_ > $#r && $eod) || $r[$_] ? $row->[$_] : () } 0..$#$row ];
+	}
+
+    return \@c;
+    } # fragment
+
 sub types
 {
     my $self = shift;
@@ -1023,6 +1096,25 @@ binary characters other than CR or NL are encountered. Note that a simple
 string like C<"\x{00a0}"> might still be binary, but not marked UTF8, so
 setting C<{ binary => 1 }> is still a wise option.
 
+=item decode_utf8
+X<decode_utf8>
+
+This attributes defaults to TRUE.
+
+While parsing, fields that are valid UTF-8, are automatically set to be
+UTF-8, so that
+
+  $csv->parse ("\xC4\xA8\n");
+
+results in
+
+  PV("\304\250"\0) [UTF8 "\x{128}"]
+
+Sometimes it might not be a desired action. To prevent those upgrades,
+set this attribute to false, and the result will be
+
+  PV("\304\250"\0)
+
 =item types
 X<types>
 
@@ -1145,6 +1237,7 @@ is equivalent to
      quote_null	           => 1,
      quote_binary          => 1,
      binary                => 0,
+     decode_utf8           => 1,
      keep_meta_info        => 0,
      allow_loose_quotes    => 0,
      allow_loose_escapes   => 0,
@@ -1344,6 +1437,52 @@ It is just a wrapper method with basic parameter checks over
 
  $csv->print ($io, [ map { $ref->{$_} } $csv->column_names ]);
 
+=head2 fragment
+
+This, for now experimental, function tries to implement RFC7111 (URI
+Fragment Identifiers for the text/csv Media Type) 1)
+
+ 1) http://tools.ietf.org/html/rfc7111
+
+ my $AoA = $csv->fragment ($io, $spec);
+
+In specifications, C<*> is used to specify the I<last> item, a dash (C<->)
+to indicate a range. All indices are 1-based: the first row or column
+has index 1. Selections on row and column can be combined with the
+semi-colon (C<;>).
+
+=over 2
+
+=item row
+
+ row=4
+ row=5-7
+ row=6-*
+ row=1-2;4;6-*
+
+=item col
+
+ col=2
+ col=1-3
+ col=4-*
+ col=1-2;4;7-*
+
+=item cell
+
+In cell-based selection, the comma (C<,>) is used to pair row and column
+
+ cell=4,1
+
+The range operator using cells can be used to define top-left and bottom-right
+cell location
+
+ cell=3,1-4,6
+
+=back
+
+RFC7111 does not allow any combination of the three selection methods. Passing
+an invalid fragment specification will croak and set error 2014.
+
 =head2 column_names
 X<column_names>
 
@@ -1607,12 +1746,19 @@ API may change in future releases.
 
 =head2 Reading a CSV file line by line:
 
- my $csv = Text::CSV_XS->new ({ binary => 1 });
+ my $csv = Text::CSV_XS->new ({ binary => 1, auto_diag => 1 });
  open my $fh, "<", "file.csv" or die "file.csv: $!";
  while (my $row = $csv->getline ($fh)) {
      # do something with @$row
      }
- $csv->eof or $csv->error_diag;
+ close $fh or die "file.csv: $!";
+
+=head3 Reading only a single column
+
+ my $csv = Text::CSV_XS->new ({ binary => 1, auto_diag => 1 });
+ open my $fh, "<", "file.csv" or die "file.csv: $!";
+ # get only the 4th column
+ my @column = map { $_->[3] } @{$csv->getline_all ($fh)};
  close $fh or die "file.csv: $!";
 
 =head2 Parsing CSV strings:
@@ -1802,7 +1948,6 @@ No guarantees, but this is what I had in mind some time ago:
 
 =item next
 
- - This might very well be 1.00
  - DIAGNOSTICS setction in pod to *describe* the errors (see below)
  - croak / carp
 
@@ -1933,6 +2078,12 @@ Self-explaining. End-of-file while inside parsing a stream. Can happen only
 when reading from streams with L</getline>, as using L</parse> is done on
 strings that are not required to have a trailing C<eol>.
 
+=item *
+2013 "ESP - Specification error for fragments RFC7111"
+X<2013>
+
+Invalid specification for URI L</fragment> specification.
+
 =item *
 2021 "EIQ - NL char inside quotes, binary off"
 X<2021>
@@ -2083,7 +2234,7 @@ ChangeLog releases 0.25 and on.
 
 =head1 COPYRIGHT AND LICENSE
 
- Copyright (C) 2007-2013 H.Merijn Brand.  All rights reserved.
+ Copyright (C) 2007-2014 H.Merijn Brand.  All rights reserved.
  Copyright (C) 1998-2001 Jochen Wiedmann. All rights reserved.
  Copyright (C) 1997      Alan Citterman.  All rights reserved.
 
